@@ -1,5 +1,7 @@
 from zipfile import ZipFile
 import datetime
+import calendar
+import json
 
 import pandas as pd
 
@@ -103,7 +105,7 @@ class GTFS:
                 dtype={
                     'route_id': str, 'service_id': str, 'trip_id': str,
                     'trip_headsign': str, 'trip_short_name': str,
-                    'direction_id': int, 'block_id': str, 'shape_id': str,
+                    'direction_id': 'Int64', 'block_id': str, 'shape_id': str,
                     'wheelchair_accessible': 'Int64', 'bikes_allowed': 'Int64'
                 })
             stop_times = pd.read_csv(
@@ -139,6 +141,8 @@ class GTFS:
                     },
                     parse_dates=['date']
                 )
+                if calendar_dates.shape[0] == 0:
+                    calendar_dates = None
             else:
                 calendar_dates = None
 
@@ -148,7 +152,7 @@ class GTFS:
                     dtype={
                         'fare_id': str, 'price': float, 'currency_type': str,
                         'payment_method': int, 'transfers': 'Int64',
-                        'agency_id': str, 'transfer_duration': int
+                        'agency_id': str, 'transfer_duration': 'Int64'
                     }
                 )
             else:
@@ -293,7 +297,7 @@ class GTFS:
                 last_date: the last date the feed is valid for
                 total_shapes (optional): the total number of shapes in the feed.
         """
-        summary = pd.Series()
+        summary = pd.Series(dtype=str)
         summary['agencies'] = self.agency.agency_name.tolist()
         summary['total_stops'] = self.stops.shape[0]
         summary['total_routes'] = self.routes.shape[0]
@@ -497,3 +501,83 @@ class GTFS:
         min_split = grouped['min'].str.split(":", expand=True).astype(int)
         grouped['diff'] = (max_split[0] - min_split[0]) + (max_split[1] - min_split[1])/60 + (max_split[2] - min_split[2])/3600
         return(grouped['diff'].sum())
+
+    def trip_distribution(self, start_date, end_date):
+        """Find the distribution of service by day of week for a given date range.
+
+        Parameters:
+            start_date (datetime.date): The start date for the search
+            end_date (datetime.date): The end date for the search
+        
+        Returns a pandas.Series containing as indices the days of the week and as values the total
+        number of trips found in the time slice.
+        """
+
+        days = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sunday']
+        dist = pd.Series(
+            index=days, 
+            name='trips',
+            dtype='int'
+            )
+
+        # Start with calendar:
+        if self.calendar is not None:
+            for dow in dist.index:
+                # Get rows where that DOW happens in the date range
+                for idx, service in self.calendar[(self.calendar[dow] == True) & (self.calendar.start_date.dt.date <= start_date) & (self.calendar.end_date.dt.date >= end_date)].iterrows():
+                    # We'll need the number of a given days of the week in that range to multiply the calendar.
+                    week = {}
+                    for i in range(((end_date + datetime.timedelta(days=1)) - start_date).days):
+                        day       = calendar.day_name[(start_date + datetime.timedelta(days=i+1)).weekday()].lower()
+                        week[day] = week[day] + 1 if day in week else 1
+
+                     # Get trips with that service id and add them to the total, only if that day is in there.
+                    if dow in week.keys():
+                        dist[dow] = dist[dow] + week[dow]*self.trips[self.trips.service_id == service.service_id].trip_id.count()
+
+        # Now check exceptions to add and remove
+        if self.calendar_dates is not None:
+            # Start by going through all the calendar dates within the date range
+            # cd = self.calendar_dates.copy()
+            for index, cd in self.calendar_dates[(self.calendar_dates['date'].dt.date >= start_date) & (self.calendar_dates['date'].dt.date <= end_date)].iterrows():
+                if cd['exception_type'] == 1:
+                    dist[days[cd['date'].dayofweek]] += self.trips[self.trips.service_id == cd['service_id']].trip_id.count()
+                else:
+                    dist[days[cd['date'].dayofweek]] -= self.trips[self.trips.service_id == cd['service_id']].trip_id.count()
+
+        return dist
+
+    def route_stops_inside(self, path_to_shape, format='geojson'):
+        """Count the number of stops a given route has inside an area
+
+        Args:
+            path_to_shape (str): 
+            format (str, optional): Format of shape. Can be 'geojson' or 'shp'. Defaults to 'geojson'.
+        """
+        from shapely.geometry import Point, shape, GeometryCollection
+        
+        count = 0
+        # For starters, let's load a bounding box and check how many stops are in the point
+        if format == 'geojson':
+            with open(path_to_shape) as f:
+                features = json.load(f)["features"]
+                boundary = GeometryCollection([shape(feature["geometry"]).buffer(0) for feature in features])
+                routes = []
+                counts = []
+                for idx, route in self.routes.iterrows():
+                    # Get all the stops on trips for that route.
+                    stops = self.stop_times[self.stop_times.trip_id.isin(self.trips[self.trips.route_id == route.route_id].trip_id)].stop_id.unique()
+                # NOTE: buffer(0) is a trick for fixing scenarios where polygons have overlapping coordinates 
+                    count = 0
+                    for idx, stop in self.stops[self.stops.stop_id.isin(stops)].iterrows():
+                        if Point(stop.stop_lon, stop.stop_lat).within(boundary):
+                            count += 1
+                    routes.append(route.route_id)
+                    counts.append(count)
+        
+        stop_count = pd.DataFrame(counts, index=routes)
+        return stop_count
+
+
+
+
